@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ class Teacher(abstract.Teacher):
 class Gui(abstract.Gui):
     def __init__(self, teacher):
         abstract.Gui.__init__(self, teacher, no_notebook=True)
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
+        self.g_music_displayer = mpd.MusicDisplayer()
         self.g_music_displayer.clear()
         self.g_music_displayer.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
         self.practise_box.pack_start(self.g_music_displayer)
