@@ -132,6 +132,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.g_config_window = None
         self.g_path_info_dlg = None
         self.g_musicviewer_window = None
+        self.m_history = []
         self.g_ui_manager = gtk.UIManager()
         self.m_action_groups = {
             'Exit': gtk.ActionGroup('Exit'),
@@ -391,8 +392,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if not self.g_path_info_dlg:
             self.g_path_info_dlg = gtk.Dialog(_("_File locations").replace("_", ""), self,
                 buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+            sc = gtk.ScrolledWindow()
+            sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+            self.g_path_info_dlg.vbox.pack_start(sc, True, True)
+            #
             vbox = gu.hig_dlg_vbox()
-            self.g_path_info_dlg.vbox.pack_start(vbox)
+            sc.add_with_viewport(vbox)
+
             box1, box2 = gu.hig_category_vbox(_("_File locations").replace("_", ""))
             vbox.pack_start(box1)
             sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -413,8 +419,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 return True
             self.g_path_info_dlg.connect('response', f)
             self.g_path_info_dlg.connect('delete-event', f)
-        else:
-            self.g_path_info_dlg.show()
+            w, h = vbox.size_request()
+            sc.set_size_request(min(w + gu.hig_dlg_vbox.border_width * 2, gtk.gdk.screen_width()*0.9), h)
     def setup_pyalsa(self, widget):
         download_pyalsa.download()
     def show_about_window(self, widget):
@@ -538,27 +544,18 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def display_user_exercises(self, w):
         col = frontpage.Column()
         page = frontpage.Page(_('User exercises'), col)
-        curdir =None
-        for filename in lessonfile.infocache.iter_user_files():
+        curdir = None
+        linklist = None
+        for filename in lessonfile.infocache.iter_user_files(only_user_collection=True):
             dir, fn = os.path.split(filename)
             if dir != curdir:
                 curdir = dir
                 linklist = frontpage.LinkList(dir)
                 col.append(linklist)
             linklist.append(filename)
-        # Move ~/.solfege/exercises/user/lesson-files first in the list
-        # since we refer to this directory in the user manual
-        for idx, c in enumerate(col):
-            if c[0].startswith(os.path.join(filesystem.user_data(), u"exercises", u"user", u"lesson-files")):
-                col.insert(0, c)
-                del col[idx+1]
-                break
-        else:
-            linklist = frontpage.LinkList(os.path.join(filesystem.user_data(), u"exercises", u"user", u"lesson-files"))
-            col.insert(0, linklist)
-
         if os.path.isdir(filesystem.user_lessonfiles()):
             linklist = None
+            col.append(frontpage.Paragraph(_('You really should move the following directory to a directory below <span font_family="monospace">%s</span>. Future versions of GNU Solfege will not display files in the old location. The user manual have details on where to place the files.') % os.path.join(filesystem.user_data(), u'exercises')))
             # Added just to be nice with people not moving their files from
             # pre 3.15.3 location:
             for filename in os.listdir(filesystem.user_lessonfiles()):
@@ -589,15 +586,18 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         self.set_title("GNU Solfege - tests")
         if not self.show_view('testspage'):
-            self.add_view(TestsView(), 'testspage')
+            p = TestsView()
+            p.connect('link-clicked', self.history_handler)
+            self.add_view(p, 'testspage')
         self.get_view().g_searchbox.show()
         if not data:
             data = solfege.app.m_frontpage_data
+        self.trim_history(self.get_view(), data)
         self.get_view().display_data(data, show_topics=show_topics)
     def on_search_all_exercises(self, widget=None):
         self.set_title("GNU Solfege")
         if not self.show_view('searchview'):
-            self.add_view(SearchView(u'Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.'), 'searchview')
+            self.add_view(SearchView(_('Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.')), 'searchview')
     def display_frontpage(self, data=None, show_topics=False):
         """
         Display the front page of the data  in solfege.app.m_frontpage_data
@@ -607,11 +607,23 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         else:
             self.set_title("GNU Solfege")
         if not self.show_view('frontpage'):
-            self.add_view(FrontPage(), 'frontpage')
+            p = FrontPage()
+            p.connect('link-clicked', self.history_handler)
+            self.add_view(p, 'frontpage')
         self.get_view().g_searchbox.show()
         if not data:
             data = solfege.app.m_frontpage_data
+        self.trim_history(self.get_view(), data)
         self.get_view().display_data(data, show_topics=show_topics)
+    def trim_history(self, new_viewer, new_page):
+        # First check if the page we want to display is in m_history.
+        # If so, we will trunkate history after it.
+        for i, (viewer, page) in enumerate(self.m_history):
+            if (new_viewer != viewer) or (new_page == page):
+                self.m_history = self.m_history[:i]
+                break
+    def history_handler(self, *w):
+        self.m_history.append(w)
     def initialise_exercise(self, teacher):
         """
         Create a Gui object for the exercise and add it to
@@ -626,6 +638,20 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             view = self.get_view()
         except KeyError:
             return
+        if (event.type == gtk.gdk.KEY_PRESS
+            and event.state == gtk.gdk.MOD1_MASK # Alt key
+            and event.keyval in (gtk.keysyms.KP_Left, gtk.keysyms.Left)
+            and self.m_history
+            and not solfege.app.m_test_mode):
+            obj, page = self.m_history[-1]
+            self.trim_history(obj, page)
+            # Find the box_dict key for obj
+            for k, o in self.box_dict.items():
+                if o == obj:
+                    obj.display_data(page)
+                    self.show_view(k)
+                    break
+            return True
         view.on_key_press_event(widget, event)
     def open_profile_manager(self, widget=None):
         p = ChangeProfileDialog(solfege.app.m_options.profile)
