@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,8 +32,6 @@ To get bar lines you have to insert a '|'
 """
 
 import logging
-import weakref
-import operator
 
 from solfege.mpd.duration import Duration
 from solfege.mpd.lexer import Lexer
@@ -85,7 +83,7 @@ class TimeSignature:
 def parse_to_score_object(music):
     lexer = Lexer(music)
     relative_mode = None
-    relto=None
+    relto = None
     transpose_pitch = None
     TOPLEVEL = 1#'toplevel'
     NOTES = 2#'notes'
@@ -178,7 +176,7 @@ def parse_to_score_object(music):
                 raise
         elif toc == '|':
             if timepos != score.get_bar_at(last_pos).end():
-                logging.warning("Bar check failed at %s" % timepos)
+                logging.warning("Bar check failed at %s", timepos)
         elif toc == '{':
             if (context == TOPLEVEL):
                 context = NOTES
@@ -210,7 +208,8 @@ def parse_to_score_object(music):
                 last_pos = timepos
                 timepos = timepos + chord_duration.get_rat_value()
                 chord_duration = None
-                relto = relto_backup; relto_backup = None
+                relto = relto_backup;
+                relto_backup = None
                 context = NOTES
             else:
                 raise ParseError("Token '>' not allowed here.", lexer)
@@ -247,7 +246,10 @@ def parse_to_score_object(music):
                 partial = None
             if context == NOTES:
                 note = elems.Note(toc_data.m_pitch, toc_data.m_duration)
-                cur_voice.append(note, stem_dir)
+                try:
+                    cur_voice.append(note, stem_dir)
+                except elems.Voice.BarFullException, e:
+                    raise ParseError(unicode(e), lexer)
                 if beam is not None:
                     beam.append(note)
                 if times is not None:
